"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  MarkdownItCrossRef: () => MarkdownItCrossRef,
  default: () => src_default
});
module.exports = __toCommonJS(src_exports);
var getAnchorName = (tokens, idx) => Number(tokens[idx].meta.id + 1).toString();
var getAnchorCaption = (tokens, idx) => tokens[idx].meta.label;
var renderReference = (tokens, idx) => {
  const id = getAnchorName(tokens, idx);
  const caption = getAnchorCaption(tokens, idx);
  return `<sup class="crossref-ref"><a href="#crossref${id}" id="crossref-ref${id}">${caption}</a></sup>`;
};
var renderOpenTag = (tokens, idx) => {
  const id = getAnchorName(tokens, idx);
  const caption = getAnchorCaption(tokens, idx);
  return `<ul class="crossref-list"><li id="crossref${id}" class="crossref-item" data-caption="${caption}">`;
};
var renderCloseTag = () => "</li>\n</ul>\n";
var crossrefDef = (state, startLine, endLine, silent) => {
  const start = state.bMarks[startLine] + state.tShift[startLine];
  const max = state.eMarks[startLine];
  if (start + 4 > max)
    return false;
  if (state.src.charCodeAt(start) !== 91)
    return false;
  if (state.src.charCodeAt(start + 1) !== 126)
    return false;
  let pos, offset;
  for (pos = start + 2; pos < max; pos++) {
    if (state.src.charCodeAt(pos) === 32)
      return false;
    if (state.src.charCodeAt(pos) === 93)
      break;
  }
  if (pos === start + 2)
    return false;
  if (pos + 1 >= max || state.src.charCodeAt(++pos) !== 58)
    return false;
  if (silent)
    return true;
  pos++;
  if (!state.env.crossrefs)
    state.env.crossrefs = {};
  if (!state.env.crossrefs.refs)
    state.env.crossrefs.refs = {};
  const label = state.src.slice(start + 2, pos - 2);
  state.env.crossrefs.refs[":" + label] = -1;
  const openToken = new state.Token("crossref_reference_open", "", 1);
  openToken.meta = { label };
  openToken.level = state.level++;
  state.tokens.push(openToken);
  const oldBMark = state.bMarks[startLine];
  const oldTShift = state.tShift[startLine];
  const oldSCount = state.sCount[startLine];
  const posAfterColon = pos;
  const initial = offset = state.sCount[startLine] + pos - (state.bMarks[startLine] + state.tShift[startLine]);
  let ch;
  while (pos < max) {
    ch = state.src.charCodeAt(pos);
    if (state.md.utils.isSpace(ch)) {
      if (ch === 9)
        offset += 4 - offset % 4;
      else
        offset++;
    } else
      break;
    pos++;
  }
  state.tShift[startLine] = pos - posAfterColon;
  state.sCount[startLine] = offset - initial;
  state.bMarks[startLine] = posAfterColon;
  state.blkIndent += 4;
  if (state.sCount[startLine] < state.blkIndent) {
    state.sCount[startLine] += state.blkIndent;
  }
  state.md.block.tokenize(state, startLine, endLine);
  state.blkIndent -= 4;
  state.tShift[startLine] = oldTShift;
  state.sCount[startLine] = oldSCount;
  state.bMarks[startLine] = oldBMark;
  const closeToken = new state.Token("crossref_reference_close", "", -1);
  closeToken.level = --state.level;
  state.tokens.push(closeToken);
  return true;
};
var crossrefRef = (state, silent) => {
  const max = state.posMax;
  const start = state.pos;
  if (start + 3 > max)
    return false;
  if (!state.env.crossrefs || !state.env.crossrefs.refs)
    return false;
  if (state.src.charCodeAt(start) !== 91)
    return false;
  if (state.src.charCodeAt(start + 1) !== 126)
    return false;
  let pos;
  for (pos = start + 2; pos < max; pos++) {
    if (state.src.charCodeAt(pos) === 32)
      return false;
    if (state.src.charCodeAt(pos) === 10)
      return false;
    if (state.src.charCodeAt(pos) === 93)
      break;
  }
  if (pos === start + 2)
    return false;
  if (pos >= max)
    return false;
  pos++;
  const label = state.src.slice(start + 2, pos - 1);
  if (typeof state.env.crossrefs.refs[":" + label] === "undefined") {
    return false;
  }
  if (!silent) {
    if (!state.env.crossrefs.list)
      state.env.crossrefs.list = [];
    let crossrefId;
    if (state.env.crossrefs.refs[":" + label] < 0) {
      crossrefId = state.env.crossrefs.list.length;
      state.env.crossrefs.list[crossrefId] = label;
      state.env.crossrefs.refs[":" + label] = crossrefId;
    } else {
      crossrefId = state.env.crossrefs.refs[":" + label];
    }
    const token = state.push("renderReference", "", 0);
    token.meta = { id: crossrefId, label };
  }
  state.pos = pos;
  state.posMax = max;
  return true;
};
var crossrefCore = (state) => {
  const list = state.env.crossrefs?.list;
  for (let i = 0; i < state.tokens.length; i++) {
    if (state.tokens[i].type === "crossref_reference_open") {
      const currentLabel = state.tokens[i].meta.label;
      const id = list ? list.findIndex((item) => item === currentLabel) : -1;
      const openToken = new state.Token("renderOpenTag", "", 1);
      openToken.meta = { id, label: currentLabel };
      state.tokens[i] = openToken;
    } else if (state.tokens[i].type === "crossref_reference_close") {
      const closeToken = new state.Token("renderCloseTag", "", -1);
      state.tokens[i] = closeToken;
    }
  }
};
var MarkdownItCrossRef = (md) => {
  md.renderer.rules.renderReference = renderReference;
  md.renderer.rules.renderOpenTag = renderOpenTag;
  md.renderer.rules.renderCloseTag = renderCloseTag;
  md.block.ruler.before("reference", "crossrefDef", crossrefDef, {
    alt: ["paragraph", "reference"]
  });
  md.inline.ruler.after("image", "crossrefRef", crossrefRef);
  md.core.ruler.after("inline", "crossrefCore", crossrefCore);
};
var src_default = MarkdownItCrossRef;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  MarkdownItCrossRef
});
